unit ituCadastrarFormulario;

{$MODE Delphi}

interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, StdCtrls, Dialogs, JvExControls, ComCtrls, ExtCtrls, Buttons,
  MaskEdit, JvBaseEdits, JvCheckBox, utuMessage, Grids, ValEdit, StarUMLintf,
  cxuCadastroDefinicaoClasse;

type

  { TitCadastrarFormulario }

  TitCadastrarFormulario = class(TForm)
    lblCodigo: TLabel;
    lblClasseName: TLabel;
    lblClassIDO: TLabel;
    lblOrdem: TLabel;
    lblObrigatorio: TLabel;
    lvCampos: TListView;
    lblExibirCampos: TLabel;
    btnUp: TSpeedButton;
    btnDown: TSpeedButton;
    edtCodigo: TEdit;
    cbbTipo: TComboBox;
    lblTipoAtributo: TLabel;
    lstAtributos: TValueListEditor;
    Panel1: TPanel;
    rbNao: TRadioButton;
    rbSim: TRadioButton;
    scrlbxTudo: TScrollBox;
    btnSalvar: TButton;
    btnCancelar: TButton;
    edtCampo: TEdit;
    lblNovoCampo: TLabel;
    btnAdd: TSpeedButton;
    procedure lstAtributosClear;
    procedure lstAtributosFill(piType: string);
    procedure btnCancelarClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure lvCamposSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure lvCamposClick(Sender: TObject);
    procedure btnSalvarClick(Sender: TObject);
    procedure cbbTipoChange(Sender: TObject);
    procedure lstAtributosValidate(Sender: TObject; ACol, ARow: Integer;
      const KeyName, KeyValue: string);
    procedure btnAddClick(Sender: TObject);
    procedure edtCampoKeyPress(Sender: TObject; var Key: Char);
    procedure rbNaoChange(Sender: TObject);
    procedure rbSimChange(Sender: TObject);

  private
    fUMLClass: IUMLClass;
    fcxCadastroDefinicaoClasse: cxCadastroDefinicaoClasse;
    fClassPackage: IUMLPackage;

    fFieldTypeList : utField;
    fTypesField: utField;
    fIdCount: Integer;

    procedure Initialize;
    procedure SendToInterface(piField: utField);
    procedure AdicionarCampo (piName : string);
  public
    Constructor Create(Sender: TComponent; piCadastroClasse: cxCadastroDefinicaoClasse; piUMLClass: IUMLClass); reintroduce;

    property FieldTypeList: utField read fFieldTypeList write fFieldTypeList;
    property UMLClass: IUMLClass read fUMLClass write fUMLClass;
    property ClassPackage: IUMLPackage read fClassPackage write fClassPackage;
  end;

  //itCadastrarFormulario = class(itInterface)
  //private
  //  fTypesField : utField;
  //  fFieldTypeList : utField;
  //  fIdCount : Integer;
  //
  //  procedure ObterAtributosSuccess (piRequest : acRequest);
  //  procedure EditarSuccess (piRequest : acRequest);
  //  procedure IncluirSuccess (piRequest : acRequest);
  //  procedure SalvarSuccess (piRequest : acRequest);
  //  { Private declarations }
  //public
  //  procedure Initialize; override;
  //  procedure Finalize; override;
  //end;

implementation

uses
  acuObject;

const
  __STRING = 1;
  __NUMBER = 2;
  __BOOLEAN = 3;

  //indices dos subitens das listviews
  __DISPLAY_NAME = 0;
  __OBRIGATORIO = 1;
  __ID = 2;
  __TIPO = 3;

{$R *.lfm}

{ itCadastrarCliente }

constructor TitCadastrarFormulario.Create(Sender: TComponent; piCadastroClasse: cxCadastroDefinicaoClasse; piUMLClass: IUMLCLass);
begin
  inherited Create(sender);

  if assigned(piUMLClass) then
  begin
    fUMLClass := piUMLClass;
    lblClassIDO.Caption := fUMLClass.GetTaggedValueAsString('DBMappings', 'DBClassMap', 'DBIDO');
    lblClasseName.Caption := fUMLClass.GetTaggedValueAsString('DBMappings', 'DBClassMap', 'Table');
  end;
  fcxCadastroDefinicaoClasse := piCadastroClasse;
  self.Initialize;
end;

procedure TitCadastrarFormulario.Initialize;
var lField, lResponseField: utField;
begin
  lField := utField.Create;
  lResponseField := utField.Create;
  fFieldTypeList := utField.Create;
  fcxCadastroDefinicaoClasse.rmObtemContexto(lField, lResponseField);
  SendToInterface(lResponseField);
  lvCamposClick(nil);
end;

procedure TitCadastrarFormulario.SendToInterface(piField: utField);
var
  lFieldItem: utField;
  lEnum : acEnumerator;
  i : Integer;
begin
  inherited;
  fTypesField := piField.FindField('FieldTypes');
  if assigned(fTypesField) then
  begin
    cbbTipo.Items.Clear;
    lEnum := fTypesField.ChildFields.GetEnumerator;
    try
      while not lEnum.EOL do  {preenche a comboBox com os tipos de field }
      begin
        lFieldItem := lEnum.Current as utField;

        cbbTipo.AddItem(lFieldItem.Name, Pointer(lFieldItem.AttributeByName('OID').AsInteger));

        lEnum.MoveNext;
      end;
    finally
      lEnum.Free;
    end;
  end;

  for i:=0 to fUMLClass.GetAttributeCount - 1 do
  begin
    AdicionarCampo(fUMLClass.GetAttributeAt(i).Name);
  end;
end;


procedure TitCadastrarFormulario.btnSalvarClick(Sender: TObject);
begin
end;

procedure TitCadastrarFormulario.AdicionarCampo (piName : string);
var
  lItem : TListItem;
begin
  lItem := lvCampos.FindCaption(0, piName, False, True, False);

  if lItem = nil then
  begin
    fIdCount := fIdCount + 1;
    fFieldTypeList.AddField(IntToStr(fIdCount));

    lItem := lvCampos.Items.Add;
    lItem.Caption := piName;
    lItem.Checked := True;

    with lItem.SubItems do
    begin
      Add(piName);             //nome do atributo
      Add('N');                //requerido
      Add(IntToStr(fIdCount)); //ID
      Add('');                 //tipo
    end;
  end
  else raise Exception.Create('Campo ''' + piName + ''' j existe.');
end;

procedure TitCadastrarFormulario.btnAddClick(Sender: TObject);
begin
  if edtCampo.Text <> '' then
  begin
    AdicionarCampo(edtCampo.Text);
    edtCampo.Clear;
  end;
end;

procedure TitCadastrarFormulario.edtCampoKeyPress(Sender: TObject; var Key: Char);
begin
  if Ord(Key) = VK_RETURN then
  begin
    btnAdd.Click;
  end;
end;

procedure TitCadastrarFormulario.btnCancelarClick(Sender: TObject);
begin
  Self.Close;
end;


procedure TitCadastrarFormulario.btnDownClick(Sender: TObject);
var
  lIndex : Integer;
begin
  lIndex := lvCampos.Selected.Index;
  //lvCampos.MoveDown(lIndex);
end;

procedure TitCadastrarFormulario.btnUpClick(Sender: TObject);
var
  lIndex : Integer;
begin
  lIndex := lvCampos.Selected.Index;
  //lvCampos.MoveUp(lIndex);
end;


procedure TitCadastrarFormulario.cbbTipoChange(Sender: TObject);
var
  lField, lTypeField : utField;
begin
  lvCampos.Selected.SubItems[__TIPO] := cbbTipo.Text;

  lField := fFieldTypeList.FieldByName(lvCampos.Selected.SubItems[__ID]);

  if lField.HasField('Type') then
    lField.RemoveField(lField.FieldByName('Type'));

  lTypeField := lField.AddField('Type');
  lTypeField.AddAttribute('name').AsString := cbbTipo.Text;
  lTypeField.AddAttribute('OID').AsInteger := Integer(cbbTipo.Items.Objects[cbbTipo.ItemIndex]);

  if cbbTipo.ItemIndex <> -1 then
  begin
     lstAtributosFill(cbbTipo.Text);
  end;
end;

procedure TitCadastrarFormulario.lstAtributosFill(piType: string);
var
  lEnum : acEnumerator;
  lFieldItem, lTypeField :utField;
  lValue, lAttribute : string;
  lIndex : Integer;
begin
  lstAtributos.Clear;
  lEnum := fTypesField.FieldByName(piType).FieldByName('Attributes').ChildFields.GetEnumerator;
  try
    while not lEnum.EOL do  {preenche a ValueList com os atributos do tipo de field }
    begin
      lFieldItem := lEnum.Current as utField;

      lAttribute := lFieldItem.Name;

      lValue := '';
      if fFieldTypeList.FieldByName(lvCampos.Selected.SubItems[__ID]).HasField('Type') then
      begin
        lTypeField := fFieldTypeList.FieldByName(lvCampos.Selected.SubItems[__ID]).FieldByName('Type');
        if lTypeField.HasField(lAttribute) then
          lValue := lTypeField.FieldByName(lAttribute).AttributeByName('value').AsString;
      end;

      lIndex := lstAtributos.InsertRow(lAttribute, lValue, True);
      lstAtributos.Strings.Objects[lIndex] := Pointer(lFieldItem.AttributeByName('OID').AsInteger);

      if lFieldItem.AttributeByName('type').AsInteger = __BOOLEAN then
      begin
        lstAtributos.ItemProps[lAttribute].EditStyle := esPickList;
        lstAtributos.ItemProps[lAttribute].PickList.Add('True');
        lstAtributos.ItemProps[lAttribute].PickList.Add('False');
      end;

      lEnum.MoveNext;
    end;
  finally
    lEnum.Free;
  end;
end;

procedure TitCadastrarFormulario.lstAtributosClear;
var
  i : Integer;
begin
  if lstAtributos.RowCount > 2 then
    for i := lstAtributos.RowCount-1 downto 1 do
      lstAtributos.DeleteRow(i)
  else
    lstAtributos.Values[lstAtributos.Keys[1]] := '';
end;


procedure TitCadastrarFormulario.lstAtributosValidate(Sender: TObject; ACol,
  ARow: Integer; const KeyName, KeyValue: string);
var
  lField, lAttributeField, lTypeField : utField;
  lOID, lAttributeType, lIntValue : Integer;
  lOK, lBoolValue: Boolean;
  lMessage : string;
begin
  if (lvCampos.SelCount > 0) and (lvCampos.Selected.SubItems[__TIPO] <> '') then
  begin
    lTypeField := fTypesField.FieldByName(lvCampos.Selected.SubItems[__TIPO]);
    lAttributeType := lTypeField.FieldByName('Attributes').FieldByName(KeyName).AttributeByName('type').AsInteger;

    lOK := True;
    if lAttributeType = __NUMBER then
    begin
      lOK := TryStrToInt(KeyValue, lIntValue);
      lMessage := 'Atributo ' + KeyName + ' deve ser do tipo INTEGER';
    end
    else if lAttributeType = __BOOLEAN then
    begin
      lOK := TryStrToBool(KeyValue, lBoolValue);
      lMessage := 'Atributo ''' + KeyName + ''' deve ser do tipo BOOLEAN';
    end;

    if KeyValue <> '' then
    begin
      if lOK then
      begin
        lOID := Integer(lstAtributos.Strings.Objects[lstAtributos.Strings.IndexOfName(KeyName)]);
        lField := fFieldTypeList.FieldByName(lvCampos.Selected.SubItems[__ID]).FieldByName('Type');
        if lField.HasField(KeyName) then
        begin
          lAttributeField := lField.FieldByName(KeyName);
          lAttributeField.AttributeByName('OID').AsInteger := lOID;
          lAttributeField.AttributeByName('value').AsString := KeyValue;
        end
        else
        begin
          lAttributeField := lField.AddField(KeyName);
          lAttributeField.AddAttribute('OID').AsInteger := lOID;
          lAttributeField.AddAttribute('value').AsString := KeyValue;
        end;
      end
      else
      begin
//          lstAtributos.Col := ACol;
//          lstAtributos.Row := ARow;
        //lstAtributos.Values[KeyName] := '';
        //MessageDlg(lMessage, mtError, [mbOK], 0);
        raise Exception.Create(lMessage);
      end;
    end
    else
    begin
      lField := fFieldTypeList.FieldByName(lvCampos.Selected.SubItems[__ID]).FieldByName('Type');
      if lField.HasField(KeyName) then
        lField.RemoveField(lField.FieldByName(KeyName));
    end;
  end;
end;


procedure TitCadastrarFormulario.rbNaoChange(Sender: TObject);
begin
  lvCampos.Selected.SubItems[__OBRIGATORIO] := 'N';
end;

procedure TitCadastrarFormulario.rbSimChange(Sender: TObject);
begin
  lvCampos.Selected.SubItems[__OBRIGATORIO] := 'S';
end;

procedure TitCadastrarFormulario.lvCamposClick(Sender: TObject);
begin
  if lvCampos.SelCount = 0 then
  begin
    rbSim.Enabled := False;
    rbNao.Enabled := False;
    btnUp.Enabled := False;
    btnDown.Enabled := False;
    cbbTipo.Enabled := False;
    lblOrdem.Enabled := False;
    lblObrigatorio.Enabled := False;
    lblTipoAtributo.Enabled := False;
    lstAtributos.Enabled := False;
    lstAtributos.Clear;
  end
  else
  begin                         
    rbSim.Enabled := True;
    rbNao.Enabled := True;
    btnUp.Enabled := True;
    btnDown.Enabled := True;
    cbbTipo.Enabled := True;
    lblOrdem.Enabled := True;
    lblObrigatorio.Enabled := True;
    lblTipoAtributo.Enabled := True;
    lstAtributos.Enabled := True;
  end;
end;

procedure TitCadastrarFormulario.lvCamposSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  if Selected then
  begin
    //edtNome.Text := Item.Caption;
    //edtNomeDisplay.Text := Item.SubItems[__DISPLAY_NAME];

    if Item.SubItems[__OBRIGATORIO] = 'N' then
      rbNao.Checked := True
    else
      rbSim.Checked := True;

    if Item.SubItems[__TIPO] = '' then
    begin
      cbbTipo.ItemIndex := -1;
      lstAtributos.Clear;
    end
    else
    begin
      cbbTipo.ItemIndex := cbbTipo.Items.IndexOf(Item.SubItems[__TIPO]);
      lstAtributosFill(Item.SubItems[__TIPO]);
    end;
  end;
end;

//procedure itCadastrarFormulario.Finalize;
//begin
//  inherited;
//  if Assigned(fTypesField) then
//    fTypesField.Free;
//end;

end.
